import * as sql from "pg-sql2";
import { PgClass } from "./plugins/PgIntrospectionPlugin";
import { GraphileResolverContext } from "graphile-build";
export { GraphileResolverContext };
declare type SQL = import("pg-sql2").SQL;
export { sql, SQL };
export interface GenContext {
    queryBuilder: QueryBuilder;
}
export declare type Gen<T> = (context: GenContext) => T;
export declare type RawAlias = symbol | string;
export declare type SQLAlias = SQL;
export declare type SQLGen = Gen<SQL> | SQL;
export declare type NumberGen = Gen<number> | number;
export declare type CursorValue = Array<unknown>;
export declare type CursorComparator = (val: CursorValue, isAfter: boolean) => void;
export declare type QueryBuilderOptions = {
    supportsJSONB?: boolean;
};
declare class QueryBuilder {
    parentQueryBuilder: QueryBuilder | void;
    context: GraphileResolverContext;
    rootValue: any;
    supportsJSONB: boolean;
    locks: {
        [a: string]: false | true | string | undefined;
    };
    finalized: boolean;
    selectedIdentifiers: boolean;
    data: {
        cursorPrefix: Array<string>;
        fixedSelectExpression: SQLGen | null | undefined;
        select: Array<[SQLGen, string]>;
        selectCursor: SQLGen | null | undefined;
        from: [SQLGen, SQLAlias] | null | undefined;
        join: Array<SQLGen>;
        where: Array<SQLGen>;
        whereBound: {
            lower: Array<SQLGen>;
            upper: Array<SQLGen>;
        };
        orderBy: Array<[SQLGen, boolean, boolean | null]>;
        orderIsUnique: boolean;
        limit: NumberGen | null | undefined;
        offset: NumberGen | null | undefined;
        first: number | null | undefined;
        last: number | null | undefined;
        beforeLock: {
            [a: string]: Array<() => void> | null;
        };
        cursorComparator: CursorComparator | null | undefined;
        liveConditions: Array<[(data?: any) => (record: any) => boolean, {
            [key: string]: SQL;
        } | undefined]>;
    };
    compiledData: {
        cursorPrefix: Array<string>;
        fixedSelectExpression: SQL | null | undefined;
        select: Array<[SQL, string]>;
        selectCursor: SQL | null | undefined;
        from: [SQL, SQLAlias] | null | undefined;
        join: Array<SQL>;
        where: Array<SQL>;
        whereBound: {
            lower: Array<SQL>;
            upper: Array<SQL>;
        };
        orderBy: Array<[SQL, boolean, boolean | null]>;
        orderIsUnique: boolean;
        limit: number | null | undefined;
        offset: number | null | undefined;
        first: number | null | undefined;
        last: number | null | undefined;
        cursorComparator: CursorComparator | null | undefined;
    };
    lockContext: {
        queryBuilder: QueryBuilder;
    };
    _children: Map<RawAlias, QueryBuilder>;
    constructor(options: QueryBuilderOptions | undefined, context: GraphileResolverContext, rootValue?: any);
    jsonbBuildObject(fields: Array<[SQL, string]>): sql.SQLQuery;
    beforeLock(field: string, fn: () => void): void;
    makeLiveCollection(_table: PgClass, cb?: (checker: (data: any) => (record: any) => boolean) => void): void;
    addLiveCondition(checkerGenerator: (data: {}) => (record: any) => boolean, requirements?: {
        [key: string]: SQL;
    }): void;
    setCursorComparator(fn: CursorComparator): void;
    addCursorCondition(cursorValue: CursorValue, isAfter: boolean): void;
    /** this method is experimental */
    fixedSelectExpression(exprGen: SQLGen): void;
    select(exprGen: SQLGen, alias: string): void;
    selectIdentifiers(table: PgClass): void;
    selectCursor(exprGen: SQLGen): void;
    from(expr: SQLGen, alias?: SQLAlias): void;
    where(exprGen: SQLGen): void;
    whereBound(exprGen: SQLGen, isLower: boolean): void;
    setOrderIsUnique(): void;
    orderBy(exprGen: SQLGen, ascending?: boolean, nullsFirst?: boolean | null): void;
    limit(limitGen: NumberGen): void;
    offset(offsetGen: NumberGen): void;
    first(first: number): void;
    last(last: number): void;
    isOrderUnique(lock?: boolean): boolean;
    getTableExpression(): SQL;
    getTableAlias(): SQL;
    getSelectCursor(): SQL | null | undefined;
    getOffset(): number;
    getFinalLimitAndOffset(): {
        limit: number | null;
        offset: number;
        flip: boolean;
    };
    getFinalOffset(): number;
    getFinalLimit(): number | null;
    getOrderByExpressionsAndDirections(): [SQL, boolean, boolean | null][];
    getSelectFieldsCount(): number;
    buildSelectFields(): SQL;
    buildSelectJson({ addNullCase, addNotDistinctFromNullCase, }: {
        addNullCase?: boolean;
        addNotDistinctFromNullCase?: boolean;
    }): sql.SQLQuery;
    buildWhereBoundClause(isLower: boolean): SQL;
    buildWhereClause(includeLowerBound: boolean, includeUpperBound: boolean, { addNullCase, addNotDistinctFromNullCase, }: {
        addNullCase?: boolean;
        addNotDistinctFromNullCase?: boolean;
    }): sql.SQLQuery;
    build(options?: {
        asJson?: boolean;
        asJsonAggregate?: boolean;
        onlyJsonField?: boolean;
        addNullCase?: boolean;
        addNotDistinctFromNullCase?: boolean;
        useAsterisk?: boolean;
    }): SQL;
    _finalize(): void;
    lock(type: string): void;
    checkLock(type: string): void;
    lockEverything(): void;
    /** this method is experimental */
    buildChild(): QueryBuilder;
    /** this method is experimental */
    buildNamedChildSelecting(name: RawAlias, from: SQLGen, selectExpression: SQLGen, alias?: SQLAlias): QueryBuilder;
    /** this method is experimental */
    getNamedChild(name: string): QueryBuilder | undefined;
}
export default QueryBuilder;
//# sourceMappingURL=QueryBuilder.d.ts.map