"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Please only use capitals for aliases and lower case for the values.
 */
exports.CREATE = "create";
exports.READ = "read";
exports.UPDATE = "update";
exports.DELETE = "delete";
exports.FILTER = "filter";
exports.ORDER = "order";
exports.ALL = "all";
exports.MANY = "many";
exports.EXECUTE = "execute";
exports.BASE = "base";
const aliases = {
    C: exports.CREATE,
    R: exports.READ,
    U: exports.UPDATE,
    D: exports.DELETE,
    F: exports.FILTER,
    O: exports.ORDER,
    A: exports.ALL,
    M: exports.MANY,
    X: exports.EXECUTE,
    B: exports.BASE,
};
const PERMISSIONS_THAT_REQUIRE_READ = [exports.UPDATE, exports.CREATE, exports.DELETE, exports.ALL, exports.MANY];
function parse(arrOrNot, errorPrefix = "Error") {
    if (!arrOrNot) {
        return null;
    }
    const arr = Array.isArray(arrOrNot)
        ? arrOrNot
        : [arrOrNot];
    let all = false;
    const arrayNormalized = [].concat(...arr.map((str) => {
        if (str === true || str === "*") {
            all = true;
            return [];
        }
        if (str[0] === ":") {
            const abbreviations = str.substr(1).split("");
            const perms = abbreviations.map((p) => aliases[p]);
            const badIndex = perms.findIndex(p => !p);
            if (badIndex >= 0) {
                throw new Error(`${errorPrefix} - abbreviated parameter '${abbreviations[badIndex]}' in '${str}' not understood`);
            }
            return perms;
        }
        else {
            const perms = str.split(",");
            // TODO: warning if not in list?
            return perms;
        }
    }));
    if (all) {
        return true;
    }
    return arrayNormalized;
}
function omit(entity, permission) {
    const tags = entity.tags;
    const omitSpecRaw = tags.omit;
    // '@include' is not being released yet because it would mean every new
    // filter we added would become a breaking change for people using @include.
    const includeSpecRaw = null;
    // const includeSpecRaw = tags.include;
    if (omitSpecRaw && includeSpecRaw) {
        throw new Error(`Error when processing instructions for ${entity.kind} '${entity.name}' - you must only specify @omit or @include, not both`);
    }
    const omitSpec = parse(omitSpecRaw, `Error when processing @omit instructions for ${entity.kind} '${entity.name}'`);
    const includeSpec = parse(includeSpecRaw, `Error when processing @include instructions for ${entity.kind} '${entity.name}'`);
    if (omitSpec) {
        if (omitSpec === true) {
            return true;
        }
        if (omitSpec.indexOf(exports.READ) >= 0) {
            const bad = PERMISSIONS_THAT_REQUIRE_READ.filter(p => omitSpec.indexOf(p) === -1);
            if (bad.length > 0) {
                throw new Error(`Processing @omit for ${entity.kind} '${entity.name}' - '${bad.join(",")}' must be omitted when '${exports.READ}' is omitted. Add '${bad.join(",")}' to the @omit clause, or use '@omit' to omit all actions.`);
            }
        }
        return omitSpec.indexOf(permission) >= 0;
    }
    else if (includeSpec) {
        if (includeSpec === true) {
            throw new Error(`Error when processing instructions for ${entity.kind} '${entity.name}' - @include should specify a list of actions`);
        }
        if (includeSpec.indexOf(exports.READ) === -1) {
            const bad = PERMISSIONS_THAT_REQUIRE_READ.find(p => includeSpec.indexOf(p) >= 0);
            if (bad) {
                throw new Error(`Error when processing @include for ${entity.kind} '${entity.name}' - we currently don't support '${bad}' when '${exports.READ}' is forbidden`);
            }
        }
        return includeSpec.indexOf(permission) === -1;
    }
    else {
        return false;
    }
}
exports.default = omit;
//# sourceMappingURL=omit.js.map