"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
const lru_1 = require("@graphile/lru");
const POSTGRAPHILE_PREPARED_STATEMENT_CACHE_SIZE = parseInt(process.env.POSTGRAPHILE_PREPARED_STATEMENT_CACHE_SIZE || "", 10) ||
    100;
let lastString;
let lastHash;
const hash = (str) => {
    if (str !== lastString) {
        lastString = str;
        lastHash = crypto_1.createHash("sha1")
            .update(str)
            .digest("base64");
    }
    return lastHash;
};
const PARSED_STATEMENTS = "parsedStatements";
const GRAPHILE_PREPARED_STATEMENT_CACHE = "_graphilePreparedStatementCache";
function pgPrepareAndRun(pgClient, text, values) {
    const connection = pgClient["connection"];
    if (!values ||
        POSTGRAPHILE_PREPARED_STATEMENT_CACHE_SIZE < 1 ||
        typeof connection !== "object" ||
        !connection ||
        typeof connection[PARSED_STATEMENTS] !== "object" ||
        !connection[PARSED_STATEMENTS]) {
        return pgClient.query(text, values);
    }
    else {
        const name = hash(text);
        if (!connection[GRAPHILE_PREPARED_STATEMENT_CACHE]) {
            connection[GRAPHILE_PREPARED_STATEMENT_CACHE] = new lru_1.default({
                maxLength: POSTGRAPHILE_PREPARED_STATEMENT_CACHE_SIZE,
                dispose(key) {
                    if (connection[PARSED_STATEMENTS][key]) {
                        pgClient
                            .query(`deallocate ${pgClient.escapeIdentifier(key)}`)
                            .then(() => {
                            delete connection[PARSED_STATEMENTS][key];
                        })
                            .catch(e => {
                            // eslint-disable-next-line no-console
                            console.error("Error releasing prepared query", e);
                        });
                    }
                },
            });
        }
        if (!connection[GRAPHILE_PREPARED_STATEMENT_CACHE].get(name)) {
            // We're relying on dispose to clear out the old ones.
            connection[GRAPHILE_PREPARED_STATEMENT_CACHE].set(name, true);
        }
        return pgClient.query({
            name,
            text,
            values,
        });
    }
}
exports.default = pgPrepareAndRun;
//# sourceMappingURL=pgPrepareAndRun.js.map