import { Plugin, Context, Build } from "graphile-build";
import { PgProc, PgType, PgClass, PgAttribute, PgConstraint, PgEntity, SmartTags } from "./PgIntrospectionPlugin";
import pgField from "./pgField";
import queryFromResolveDataFactory from "../queryFromResolveDataFactory";
import addStartEndCursor from "./addStartEndCursor";
import baseOmit from "../omit";
import makeProcField from "./makeProcField";
import parseIdentifier from "../parseIdentifier";
import viaTemporaryTable from "./viaTemporaryTable";
import pgPrepareAndRun from "../pgPrepareAndRun";
import { PoolClient } from "pg";
export declare type PgTypeModifier = string | number | null;
/**
 * Given a PgEntity and a tag name, fetches the tag with that name. If it
 * exists, asserts that it is a string (and not a boolean, or array). If it
 * doesn't exist, returns null.
 */
export declare function stringTag(pgEntity: PgEntity, tagName: string): string | null;
declare type PgColumnFilterFunction = (attr: PgAttribute, build: Build, context: Context) => boolean;
declare type PgConfig = import("pg").Pool | import("pg").PoolClient | string;
declare module "graphile-build" {
    interface GraphileResolverContext {
        pgClient: PoolClient;
    }
    interface GraphileBuildOptions {
        pgConfig: PgConfig;
        pgSchemas: Array<string>;
        persistentMemoizeWithKey?: <T>(key: string, fn: () => T) => T;
        pgDisableDefaultMutations?: boolean;
        pgSimpleCollections?: "only" | "both" | "omit";
        pgStrictFunctions?: boolean;
        pgColumnFilter?: PgColumnFilterFunction;
        pgIgnoreRBAC?: boolean;
        pgIgnoreIndexes?: boolean;
        pgHideIndexWarnings?: boolean;
        pgLegacyJsonUuid?: boolean;
    }
    interface Build {
        pgSql: typeof import("pg-sql2");
        graphileBuildPgVersion: string;
        pgStrictFunctions: boolean;
        pgColumnFilter: PgColumnFilterFunction;
        pgQueryFromResolveData: ReturnType<typeof queryFromResolveDataFactory>;
        pgAddStartEndCursor: typeof addStartEndCursor;
        pgOmit: typeof baseOmit;
        pgMakeProcField: typeof makeProcField;
        pgParseIdentifier: typeof parseIdentifier;
        pgViaTemporaryTable: typeof viaTemporaryTable;
        describePgEntity: typeof describePgEntity;
        pgField: typeof pgField;
        sqlCommentByAddingTags: typeof sqlCommentByAddingTags;
        pgPrepareAndRun: typeof pgPrepareAndRun;
    }
    interface Inflection {
        conditionType(typeName: string): string;
        inputType(typeName: string): string;
        rangeBoundType(typeName: string): string;
        rangeType(typeName: string): string;
        patchType(typeName: string): string;
        baseInputType(typeName: string): string;
        patchField(itemName: string): string;
        orderByType(typeName: string): string;
        edge(typeName: string): string;
        connection(typeName: string): string;
        _functionName(proc: PgProc): string;
        _typeName(type: PgType): string;
        _tableName(table: PgClass): string;
        _singularizedTableName(table: PgClass): string;
        _columnName(attr: PgAttribute, _options?: {
            skipRowId?: boolean;
        }): string;
        enumType(type: PgType): string;
        argument(name: string | null | undefined, index: number): string;
        orderByEnum(columnName: string, ascending: boolean): string;
        orderByColumnEnum(attr: PgAttribute, ascending: boolean): string;
        orderByComputedColumnEnum(pseudoColumnName: string, proc: PgProc, table: PgClass, ascending: boolean): string;
        domainType(type: PgType): string;
        enumName(inValue: string): string;
        tableNode(table: PgClass): string;
        tableFieldName(table: PgClass): string;
        allRows(table: PgClass): string;
        allRowsSimple(table: PgClass): string;
        functionMutationName(proc: PgProc): string;
        functionMutationResultFieldName(proc: PgProc, gqlType: import("graphql").GraphQLNamedType, plural?: boolean, outputArgNames?: Array<string>): string;
        functionQueryName(proc: PgProc): string;
        functionQueryNameList(proc: PgProc): string;
        functionPayloadType(proc: PgProc): string;
        functionInputType(proc: PgProc): string;
        functionOutputFieldName(proc: PgProc, outputArgName: string, index: number): string;
        tableType(table: PgClass): string;
        column(attr: PgAttribute): string;
        computedColumn(pseudoColumnName: string, proc: PgProc, _table: PgClass): string;
        computedColumnList(pseudoColumnName: string, proc: PgProc, _table: PgClass): string;
        singleRelationByKeys(detailedKeys: PgAttribute[], table: PgClass, _foreignTable: PgClass, constraint: PgConstraint): string;
        singleRelationByKeysBackwards(detailedKeys: PgAttribute[], table: PgClass, _foreignTable: PgClass, constraint: PgConstraint): string;
        manyRelationByKeys(detailedKeys: PgAttribute[], table: PgClass, _foreignTable: PgClass, constraint: PgConstraint): string;
        manyRelationByKeysSimple(detailedKeys: PgAttribute[], table: PgClass, _foreignTable: PgClass, constraint: PgConstraint): string;
        rowByUniqueKeys(detailedKeys: PgAttribute[], table: PgClass, constraint: PgConstraint): string;
        updateByKeys(detailedKeys: PgAttribute[], table: PgClass, constraint: PgConstraint): string;
        deleteByKeys(detailedKeys: PgAttribute[], table: PgClass, constraint: PgConstraint): string;
        updateByKeysInputType(detailedKeys: PgAttribute[], table: PgClass, constraint: PgConstraint): string;
        deleteByKeysInputType(detailedKeys: PgAttribute[], table: PgClass, constraint: PgConstraint): string;
        updateNode(table: PgClass): string;
        deleteNode(table: PgClass): string;
        deletedNodeId(table: PgClass): string;
        updateNodeInputType(table: PgClass): string;
        deleteNodeInputType(table: PgClass): string;
        edgeField(table: PgClass): string;
        recordFunctionReturnType(proc: PgProc): string;
        recordFunctionConnection(proc: PgProc): string;
        recordFunctionEdge(proc: PgProc): string;
        scalarFunctionConnection(proc: PgProc): string;
        scalarFunctionEdge(proc: PgProc): string;
        createField(table: PgClass): string;
        createInputType(table: PgClass): string;
        createPayloadType(table: PgClass): string;
        updatePayloadType(table: PgClass): string;
        deletePayloadType(table: PgClass): string;
    }
}
export declare function preventEmptyResult<O>(obj: O): O;
export declare function describePgEntity(entity: PgEntity, includeAlias?: boolean): string;
declare function sqlCommentByAddingTags(entity: PgEntity, tagsToAdd: SmartTags): string;
declare const _default: Plugin;
export default _default;
//# sourceMappingURL=PgBasicsPlugin.d.ts.map