import { Plugin } from "graphile-build";
declare module "graphile-build" {
    interface GraphileBuildOptions {
        pgEnableTags?: boolean;
        pgThrowOnMissingSchema?: boolean;
        pgIncludeExtensionResources?: boolean;
        pgLegacyFunctionsOnly?: boolean;
        pgSkipInstallingWatchFixtures?: boolean;
        pgAugmentIntrospectionResults?: (introspectionResult: RawishIntrospectionResults) => RawishIntrospectionResults;
        pgOwnerConnectionString?: string;
    }
    interface Build {
        pgIntrospectionResultsByKind: PgIntrospectionResultsByKind;
    }
}
export declare enum PgEntityKind {
    NAMESPACE = "namespace",
    PROCEDURE = "procedure",
    CLASS = "class",
    TYPE = "type",
    ATTRIBUTE = "attribute",
    CONSTRAINT = "constraint",
    EXTENSION = "extension",
    INDEX = "index"
}
export declare type SmartTagValue = true | string | Array<string>;
export declare type SmartTags = {
    [tag: string]: SmartTagValue;
};
export interface PgNamespace {
    kind: PgEntityKind.NAMESPACE;
    id: string;
    name: string;
    comment: string | void;
    description: string | void;
    tags: SmartTags;
}
export interface PgProc {
    kind: PgEntityKind.PROCEDURE;
    id: string;
    name: string;
    comment: string | void;
    description: string | void;
    namespaceId: string;
    namespaceName: string;
    isStrict: boolean;
    returnsSet: boolean;
    isStable: boolean;
    returnTypeId: string;
    argTypeIds: Array<string>;
    argNames: Array<string>;
    argModes: Array<"i" | "o" | "b" | "v" | "t">;
    inputArgsCount: number;
    argDefaultsNum: number;
    namespace: PgNamespace;
    tags: SmartTags;
    cost: number;
    aclExecutable: boolean;
    language: string;
}
export interface PgClass {
    kind: PgEntityKind.CLASS;
    id: string;
    name: string;
    comment: string | void;
    description: string | void;
    classKind: string;
    namespaceId: string;
    namespaceName: string;
    typeId: string;
    isSelectable: boolean;
    isInsertable: boolean;
    isUpdatable: boolean;
    isDeletable: boolean;
    isExtensionConfigurationTable: boolean;
    namespace: PgNamespace;
    type: PgType;
    tags: SmartTags;
    attributes: Array<PgAttribute>;
    constraints: Array<PgConstraint>;
    foreignConstraints: Array<PgConstraint>;
    primaryKeyConstraint: PgConstraint | void;
    aclSelectable: boolean;
    aclInsertable: boolean;
    aclUpdatable: boolean;
    aclDeletable: boolean;
    canUseAsterisk: boolean;
}
export interface PgType {
    kind: PgEntityKind.TYPE;
    id: string;
    name: string;
    comment: string | void;
    description: string | void;
    namespaceId: string;
    namespaceName: string;
    type: string;
    category: string;
    domainIsNotNull: boolean;
    arrayItemTypeId: string | void;
    arrayItemType: PgType | void;
    arrayType: PgType | void;
    typeLength: number | void;
    isPgArray: boolean;
    classId: string | void;
    class: PgClass | void;
    domainBaseTypeId: string | void;
    domainBaseType: PgType | void;
    domainTypeModifier: number | void;
    domainHasDefault: boolean;
    enumVariants: Array<string> | null;
    rangeSubTypeId: string | null;
    tags: SmartTags;
}
export interface PgAttribute {
    kind: PgEntityKind.ATTRIBUTE;
    classId: string;
    num: number;
    name: string;
    comment: string | void;
    description: string | void;
    typeId: string;
    typeModifier: number;
    isNotNull: boolean;
    hasDefault: boolean;
    identity: "" | "a" | "d";
    class: PgClass;
    type: PgType;
    namespace: PgNamespace;
    tags: SmartTags;
    aclSelectable: boolean;
    aclInsertable: boolean;
    aclUpdatable: boolean;
    isIndexed: boolean | void;
    isUnique: boolean | void;
    columnLevelSelectGrant: boolean;
}
export interface PgConstraint {
    kind: PgEntityKind.CONSTRAINT;
    id: string;
    name: string;
    type: string;
    classId: string;
    class: PgClass;
    foreignClassId: string | void;
    foreignClass: PgClass | void;
    comment: string | void;
    description: string | void;
    keyAttributeNums: Array<number>;
    keyAttributes: Array<PgAttribute>;
    foreignKeyAttributeNums: Array<number>;
    foreignKeyAttributes: Array<PgAttribute>;
    namespace: PgNamespace;
    isIndexed: boolean | void;
    tags: SmartTags;
    isFake?: boolean;
}
export interface PgExtension {
    kind: PgEntityKind.EXTENSION;
    id: string;
    name: string;
    namespaceId: string;
    namespaceName: string;
    relocatable: boolean;
    version: string;
    configurationClassIds?: Array<string>;
    comment: string | void;
    description: string | void;
    tags: SmartTags;
}
export interface PgIndex {
    kind: PgEntityKind.INDEX;
    id: string;
    name: string;
    namespaceName: string;
    classId: string;
    class: PgClass;
    numberOfAttributes: number;
    indexType: string;
    isUnique: boolean;
    isPrimary: boolean;
    isPartial: boolean;
    attributeNums: Array<number>;
    attributePropertiesAsc: Array<boolean> | void;
    attributePropertiesNullsFirst: Array<boolean> | void;
    description: string | void;
    comment: string | void;
    tags: SmartTags;
}
export declare type PgEntity = PgNamespace | PgProc | PgClass | PgType | PgAttribute | PgConstraint | PgExtension | PgIndex;
export declare type PgIntrospectionResultsByKind = {
    __pgVersion: number;
    attribute: PgAttribute[];
    attributeByClassIdAndNum: {
        [classId: string]: {
            [num: string]: PgAttribute;
        };
    };
    class: PgClass[];
    classById: {
        [classId: string]: PgClass;
    };
    constraint: PgConstraint[];
    extension: PgExtension[];
    extensionById: {
        [extId: string]: PgExtension;
    };
    index: PgIndex[];
    namespace: PgNamespace[];
    namespaceById: {
        [namespaceId: string]: PgNamespace;
    };
    procedure: PgProc[];
    type: PgType[];
    typeById: {
        [typeId: string]: PgType;
    };
};
declare type RawishIntrospectionResults = Pick<PgIntrospectionResultsByKind, "__pgVersion" | "namespace" | "class" | "attribute" | "type" | "constraint" | "procedure" | "extension" | "index">;
declare const _default: Plugin;
export default _default;
//# sourceMappingURL=PgIntrospectionPlugin.d.ts.map