import { Plugin, ResolvedLookAhead } from "graphile-build";
import { PgEntity, PgClass, PgConstraint, PgType } from "./PgIntrospectionPlugin";
import { PgTypeModifier } from "./PgBasicsPlugin";
import { SQL } from "../QueryBuilder";
interface GqlTypeByTypeIdAndModifier {
    [typeId: string]: {
        [modifier: string]: import("graphql").GraphQLOutputType;
    };
}
interface GqlInputTypeByTypeIdAndModifier {
    [typeId: string]: {
        [modifier: string]: import("graphql").GraphQLInputType;
    };
}
declare type FragmentTweaker = (fragment: SQL, resolveData: ResolvedLookAhead) => SQL;
declare type TypeGen<T> = (set: (type: T) => void, typeModifier: PgTypeModifier) => void | null | T;
declare type Pg2GqlMapper = {
    [id: string]: {
        map: (valueFromDatabase: any) => any;
        unmap: (valueFromGraphQL: any, modifier?: PgTypeModifier) => SQL;
    };
};
declare module "graphile-build" {
    interface Build {
        pgGqlTypeByTypeIdAndModifier?: GqlTypeByTypeIdAndModifier;
        pgGqlInputTypeByTypeIdAndModifier?: GqlInputTypeByTypeIdAndModifier;
        pgRegisterGqlTypeByTypeId(typeId: string, gen: TypeGen<import("graphql").GraphQLOutputType>, yieldToExisting?: boolean): void;
        pgRegisterGqlInputTypeByTypeId(typeId: string, gen: TypeGen<import("graphql").GraphQLInputType>, yieldToExisting?: boolean): void;
        pgGetGqlTypeByTypeIdAndModifier(typeId: string, typeModifier: PgTypeModifier, useFallback?: boolean): import("graphql").GraphQLOutputType | null;
        pgGetGqlInputTypeByTypeIdAndModifier(typeId: string, typeModifier: PgTypeModifier, useFallback?: boolean): import("graphql").GraphQLInputType | null;
        pg2GqlMapper: Pg2GqlMapper;
        pg2gql: (val: any, type: PgType) => any;
        pg2gqlForType: (type: PgType) => (value: any) => any;
        gql2pg: (val: any, type: PgType, modifier?: PgTypeModifier) => SQL;
        pgTweakFragmentForTypeAndModifier: (fragment: SQL, type: PgType, typeModifier: PgTypeModifier, resolveData: ResolvedLookAhead) => SQL;
        pgTweaksByTypeId: {
            [typeId: string]: FragmentTweaker;
        };
        pgTweaksByTypeIdAndModifer: {
            [modifier: string]: {
                [typeId: string]: FragmentTweaker;
            };
        };
    }
    interface GraphileBuildOptions {
        pgExtendedTypes?: boolean;
        pgSkipHstore?: boolean;
        pgUseCustomNetworkScalars?: boolean;
        disableIssue390Fix?: boolean;
    }
    interface ScopeGraphQLObjectType {
        pgIntrospection?: PgEntity;
        pgIntrospectionTable?: PgClass;
        pgSubtypeIntrospection?: PgEntity;
        pgTypeModifier?: PgTypeModifier;
        isIntervalType?: boolean;
        isPointType?: boolean;
        isPgRangeType?: boolean;
        isPgRangeBoundType?: boolean;
    }
    interface ScopeGraphQLObjectTypeFieldsField {
        pgFieldIntrospection?: PgEntity;
        pgFieldIntrospectionTable?: PgClass;
        pgFieldConstraint?: PgConstraint;
        isPgFieldConnection?: boolean;
        isPgFieldSimpleCollection?: boolean;
        isCursorField?: boolean;
    }
    interface ScopeGraphQLInputObjectType {
        pgIntrospection?: PgEntity;
        pgSubtypeIntrospection?: PgEntity;
        pgTypeModifier?: PgTypeModifier;
        isIntervalInputType?: boolean;
        isPointInputType?: boolean;
        isPgRangeInputType?: boolean;
        isPgRangeBoundInputType?: boolean;
    }
    interface ScopeGraphQLInputObjectTypeFieldsField {
        pgFieldIntrospection?: PgEntity;
    }
    interface ScopeGraphQLEnumType {
        pgIntrospection?: PgEntity;
        isPgEnumType?: boolean;
        isPgRowSortEnum?: boolean;
    }
    interface Inflection {
        hstoreType(): string;
    }
}
declare const _default: Plugin;
export default _default;
//# sourceMappingURL=PgTypesPlugin.d.ts.map