"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = require("debug");
const chalk_1 = require("chalk");
function formatSQLForDebugging(sql) {
    let colourIndex = 0;
    const allowedColours = [
        chalk_1.default.red,
        chalk_1.default.green,
        chalk_1.default.yellow,
        chalk_1.default.blue,
        chalk_1.default.magenta,
        chalk_1.default.cyan,
        chalk_1.default.white,
        chalk_1.default.black,
    ];
    function nextColor() {
        colourIndex = (colourIndex + 1) % allowedColours.length;
        return allowedColours[colourIndex];
    }
    const colours = {};
    /* Yep - that's `colour` from English and `ize` from American */
    function colourize(str) {
        if (!colours[str]) {
            colours[str] = nextColor();
        }
        return colours[str].bold.call(null, str);
    }
    let indentLevel = 0;
    function handleIndent(_all, rawMatch) {
        const match = rawMatch.replace(/ $/, "");
        if (match === "(") {
            indentLevel++;
            return match + "\n" + "  ".repeat(indentLevel);
        }
        else if (match === ")") {
            indentLevel--;
            return "\n" + "  ".repeat(indentLevel) + match;
        }
        else if (match === "),") {
            indentLevel--;
            return ("\n" +
                "  ".repeat(indentLevel) +
                match +
                "\n" +
                "  ".repeat(indentLevel));
        }
        else if (match === ",") {
            return match + "\n" + "  ".repeat(indentLevel);
        }
        else {
            return "\n" + "  ".repeat(indentLevel) + match.replace(/^\s+/, "");
        }
    }
    const tidySql = sql
        .replace(/\s+/g, " ")
        .replace(/\s+(?=$|\n|\))/g, "")
        .replace(/(\n|^|\()\s+/g, "$1")
        .replace(/(\(|\)|\), ?|, ?| (select|insert|update|delete|from|where|and|or|order|limit)(?= ))/g, handleIndent)
        .replace(/\(\s*([A-Za-z0-9_."' =]{1,50})\s*\)/g, "($1)")
        .replace(/\(\s*(\([A-Za-z0-9_."' =]{1,50}\))\s*\)/g, "($1)")
        .replace(/\n\s*and \(TRUE\)/g, chalk_1.default.gray(" and (TRUE)"));
    const colouredSql = tidySql.replace(/__local_[0-9]+__/g, colourize);
    return colouredSql;
}
exports.formatSQLForDebugging = formatSQLForDebugging;
const rawDebugSql = debug_1.default("graphile-build-pg:sql");
function debugSql(sql) {
    if (!rawDebugSql.enabled) {
        return;
    }
    rawDebugSql("%s", "\n" + formatSQLForDebugging(sql));
}
Object.assign(debugSql, rawDebugSql);
exports.default = debugSql;
//# sourceMappingURL=debugSql.js.map